@@ -8,20 +8,21 @@ maps with pattern matching.
 Usage:
     chromosight detect  [--kernel-config=FILE] [--pattern=loops]
                         [--pearson=auto] [--win-size=auto] [--iterations=auto]
-                        [--win-fmt={json,npy}] [--force-norm]
+                        [--win-fmt={json,npy}] [--norm={auto,raw,force}]
                         [--subsample=no] [--inter] [--tsvd] [--smooth-trend]
                         [--n-mads=5] [--min-dist=0] [--max-dist=auto]
                         [--no-plotting] [--min-separation=auto] [--dump=DIR]
                         [--threads=1] [--perc-zero=auto]
                         [--perc-undetected=auto] <contact_map> <prefix>
     chromosight generate-config [--preset loops] [--click contact_map]
-                        [--force-norm] [--win-size=auto] [--n-mads=5]
-                        [--threads=1] <prefix>
+                        [--norm={auto,raw,norm}] [--win-size=auto] [--n-mads=5]
+                        [--chroms=CHROMS] [--inter] [--threads=1] <prefix>
     chromosight quantify [--inter] [--pattern=loops] [--subsample=no]
-                         [--win-fmt=json] [--kernel-config=FILE] [--force-norm]
+                         [--win-fmt=json] [--kernel-config=FILE] [--norm={auto,raw,norm}]
                          [--threads=1] [--n-mads=5] [--win-size=auto]
                          [--perc-undetected=auto] [--perc-zero=auto]
                          [--no-plotting] [--tsvd] <bed2d> <contact_map> <prefix>
+    chromosight list-kernels [--long] [--mat] [--name=kernel_name]
     chromosight test
 
     detect:
@@ -36,12 +37,12 @@ Usage:
         Given a list of pairs of positions and a contact map, computes the
         correlation coefficients between those positions and the kernel of the
         selected pattern.
+    list-kernels:
+        Prints information about available kernels.
     test:
         Download example data and run loop detection on it.
 
 Arguments for detect:
-    -h, --help                  Display this help message.
-    --version                   Display the program's current version.
     contact_map                 The Hi-C contact map to detect patterns on, in
                                 bedgraph2d or cool format.
     prefix                      Common path prefix used to generate output files.
@@ -64,14 +65,28 @@ Arguments for generate-config:
     -e, --preset=loops          Generate a preset config for the given pattern.
                                 Preset configs available are "loops" and
                                 "borders". [default: loops]
-    -c, --click contact_map     Show input contact map and uses double clicks from
+    -c, --click=contact_map     Show input contact map and uses double clicks from
                                 user to build the kernel. Warning: memory-heavy,
                                 reserve for small genomes or subsetted matrices.
+    -C, --chroms=CHROMS         Comma-separated list of chromosome names. When used
+                                with --click, this will show each chromosome's
+                                one-by-one sequentially instead of the whole genome.
+                                This is useful to reduce memory usage.
+
+Arguments for list-kernels:
+    --name=kernel_name      Only show information related to a particular
+                            kernel.[default: all]
+    --long                  Show default parameters in addition to kernel names.
+    --mat                   Prints an ascii representation of the kernel matrix.
 
 Basic options:
-    -F, --force-norm            Re-compute matrix normalization (balancing) and
-                                overwrite weights present in the cool files instead
-                                of reusing them.
+    -h, --help                  Display this help message.
+    --version                   Display the program's current version.
+    --verbose                   Displays the logo.
+    -n, --norm={auto,raw,force} Normalization / balancing behaviour. auto: weights
+                                present in the cool file are used. raw: raw contact
+                                values are used. force: recompute weights and
+                                overwrite existing values. raw[default: auto]
     -I, --inter                 Enable to consider interchromosomal contacts.
                                 Warning: Experimental feature with high memory
                                 consumption, only use with small matrices.
@@ -81,7 +96,7 @@ Basic options:
                                 for detection. [default: auto]
     -P, --pattern=loops         Which pattern to detect. This will use preset
                                 configurations for the given pattern. Possible
-                                values are: loops, borders, hairpins and
+                                values are: loops, loops_small, borders, hairpins and
                                 centromeres. [default: loops]
     -p, --pearson=auto          Pearson correlation threshold when detecting patterns
                                 in the contact map. Lower values leads to potentially
@@ -107,7 +122,7 @@ Advanced options:
                                 config path. Use this to override pattern if
                                 you do not want to use one of the preset
                                 patterns.
-    -n, --no-plotting           Disable generation of pileup plots.
+    --no-plotting               Disable generation of pileup plots.
     -N, --n-mads=5              Maximum number of median absolute deviations below
                                 the median of the bin sums distribution allowed to
                                 consider detectable bins. [default: 5]
@@ -135,26 +150,34 @@ Advanced options:
 
 
 """
-import numpy as np
-import pandas as pd
+import sys
 import os
 import io
+import itertools as it
 from contextlib import contextmanager
-import sys
 import json
-import docopt
 import tempfile
+import pathlib
+import numpy as np
+import pandas as pd
+import docopt
 import multiprocessing as mp
 from chromosight.version import __version__
 from chromosight.utils.contacts_map import HicGenome
 import chromosight.utils.io as cio
 import chromosight.utils.detection as cid
-from chromosight.utils.plotting import pileup_plot, click_finder
+from chromosight.utils.plotting import (
+    pileup_plot,
+    click_finder,
+    print_ascii_mat,
+)
 from chromosight.utils.preprocessing import resize_kernel
 from chromosight.utils.stats import fdr_correction
+import chromosight.kernels as ck
 import scipy.ndimage as ndi
 import matplotlib.pyplot as plt
 
+LOGO = np.loadtxt(pathlib.Path(__file__).parents[0] / "logo.txt")
 URL_EXAMPLE_DATASET = (
     "https://raw.githubusercontent.com/koszullab/"
     "chromosight/master/data_test/example.cool"
@@ -172,6 +195,8 @@ Matrix already balanced, reusing weights
 Preprocessing sub-matrices...
 Detecting patterns...
 89 patterns detected
+Saving patterns in chromosight_test.tsv
+Saving patterns in chromosight_test.json
 """
 
 
@@ -202,6 +227,72 @@ def _override_kernel_config(param_name, param_value, param_type, config):
     return config
 
 
+def _quantify_sub_mat(data):
+    """
+    This function is dispatched by multiprocessing to
+    run the quantification pipeline.
+    """
+    sub = data[0][1]
+    config = data[1]
+    kernel = data[2]
+    positions = data[3]
+    # If there is no pattern on this sub matrix,
+    # do not scan it.
+    if positions.shape[0]:
+        sub.contact_map.create_mat()
+        # Feed the submatrix to quantification pipeline
+        patterns, windows = cid.pattern_detector(
+            sub.contact_map,
+            config,
+            kernel,
+            coords=np.array(positions.loc[:, ["bin1", "bin2"]]),
+            full=True,
+            tsvd=config["tsvd"],
+        )
+        sub.contact_map.destroy_mat()
+    else:
+        patterns = windows = None
+
+    return {
+        "coords": patterns,
+        "windows": windows,
+        "chr1": sub.chr1,
+        "chr2": sub.chr2,
+    }
+
+
+def _get_chrom_pos(positions, hic_genome, chr1, chr2):
+    """
+    Helper function to filter input 2D genomic positions
+    and convert them from whole genome to submatrix coords.
+    """
+    # Filter patterns falling onto this sub-matrix
+    sub_pat = positions.loc[
+        (positions.chrom1 == chr1) & (positions.chrom2 == chr2)
+    ]
+    # Convert genomic coordinates to bins for horizontal and vertical axes
+    for ax in [1, 2]:
+        sub_pat_ax = sub_pat.loc[:, [f"chrom{ax}", f"pos{ax}"]].rename(
+            columns={f"chrom{ax}": "chrom", f"pos{ax}": "pos"}
+        )
+        sub_pat_bins = hic_genome.coords_to_bins(sub_pat_ax)
+        sub_pat[f"bin{ax}"] = sub_pat_bins
+
+    # Check for nan bins (coords that do not match any Hi-C fragments
+    fall_out = np.isnan(sub_pat["bin1"]) | np.isnan(sub_pat["bin2"])
+    if np.any(fall_out):
+        n_out = len(sub_pat_bins[fall_out])
+        sys.stderr.write(
+            f"\n{n_out} entr{'ies' if n_out > 1 else 'y'} outside "
+            "genomic coordinates of the Hi-C matrix will be ignored.\n"
+        )
+        sub_pat = sub_pat.loc[~fall_out, :]
+    sub_pat_idx = sub_pat.index.values
+    # Convert bins from whole genome matrix to sub matrix
+    sub_pat = hic_genome.get_sub_mat_pattern(chr1, chr2, sub_pat)
+    return sub_pat_idx, sub_pat
+
+
 def cmd_quantify(args):
     bed2d_path = args["<bed2d>"]
     mat_path = args["<contact_map>"]
@@ -214,7 +305,7 @@ def cmd_quantify(args):
     perc_undetected = args["--perc-undetected"]
     plotting_enabled = False if args["--no-plotting"] else True
     threads = int(args["--threads"])
-    force_norm = args["--force-norm"]
+    norm = args["--norm"]
     tsvd = 0.999 if args["--tsvd"] else None
     win_fmt = args["--win-fmt"]
     if win_fmt not in ["npy", "json"]:
@@ -255,6 +346,7 @@ def cmd_quantify(args):
     max_diag = hic_genome.clr.shape[0] * hic_genome.clr.binsize
     cfg["max_dist"] = min(furthest, max_diag)
     cfg["min_dist"] = 0
+    cfg["tsvd"] = tsvd
     cfg = _override_kernel_config("max_perc_zero", perc_zero, float, cfg)
     cfg = _override_kernel_config(
         "max_perc_undetected", perc_undetected, float, cfg
@@ -263,13 +355,14 @@ def cmd_quantify(args):
     # Notify contact map instance of changes in scanning distance
     hic_genome.kernel_config = cfg
     # Normalize (balance) matrix using ICE
-    hic_genome.normalize(force_norm=force_norm, n_mads=n_mads, threads=threads)
+    hic_genome.normalize(norm=norm, n_mads=n_mads, threads=threads)
     # Initialize output structures
     bed2d["score"] = np.nan
     bed2d["pvalue"] = np.nan
     positions = bed2d.copy()
     # Only resize kernel matrix if explicitely requested
     km, kn = cfg["kernels"][0].shape
+    n_kernels = len(cfg['kernels'])
     if win_size != "auto":
         if not win_size % 2:
             raise ValueError("--win-size must be odd")
@@ -284,128 +377,124 @@ def cmd_quantify(args):
     # matrix is processed on the fly (obs / exp, trimming diagonals > max dist)
     hic_genome.make_sub_matrices()
     windows = np.full((positions.shape[0], km, kn), np.nan)
+    # We will store a copy of coordinates for each kernel
+    bed2d_out = [bed2d.copy() for _ in range(n_kernels)]
+    windows_out = [windows.copy() for _ in range(n_kernels)]
     # For each position, we use the center of the BED interval
     positions["pos1"] = (positions.start1 + positions.end1) // 2
     positions["pos2"] = (positions.start2 + positions.end2) // 2
     # Use each kernel matrix available for the pattern
     for kernel_id, kernel_matrix in enumerate(cfg["kernels"]):
         cio.progress(kernel_id, len(cfg["kernels"]), f"Kernel: {kernel_id}\n")
-        # Iterate over intra- and inter-chromosomal sub-matrices
         n_sub_mats = hic_genome.sub_mats.shape[0]
-        for sub_mat_id, sub_mat in enumerate(hic_genome.sub_mats.iterrows()):
-            mat = sub_mat[1]
-            cio.progress(sub_mat_id, n_sub_mats, f"{mat.chr1}-{mat.chr2}")
-            mat.contact_map.create_mat()
-            # Filter patterns falling onto this sub-matrix
-            sub_pat = positions.loc[
-                (positions.chrom1 == mat.chr1) & (positions.chrom2 == mat.chr2)
-            ]
-            # Convert genomic coordinates to bins for horizontal and vertical axes
-            for ax in [1, 2]:
-                sub_pat_ax = sub_pat.loc[:, [f"chrom{ax}", f"pos{ax}"]].rename(
-                    columns={f"chrom{ax}": "chrom", f"pos{ax}": "pos"}
-                )
-                sub_pat_bins = hic_genome.coords_to_bins(sub_pat_ax)
-                sub_pat[f"bin{ax}"] = sub_pat_bins
+        # Retrieve input positions for each submatrix and convert
+        # coordinates from whole genome to submatrix.
+        sub_pos = [
+            _get_chrom_pos(positions, hic_genome, m[1].chr1, m[1].chr2)
+            for m in hic_genome.sub_mats.iterrows()
+        ]
+        # Apply quantification procedure to all sub matrices in parallel
+        sub_mat_data = zip(
+            hic_genome.sub_mats.iterrows(),
+            [cfg for _ in range(n_sub_mats)],
+            [kernel_matrix for _ in range(n_sub_mats)],
+            [s[1] for s in sub_pos],
+        )
+        # Run quantification in parallel on different sub matrices,
+        # and show progress when gathering results
+        sub_mat_results = []
+        # Run in multiprocessing subprocesses
+        if threads > 1:
+            pool = mp.Pool(threads)
+            dispatcher = pool.imap(_quantify_sub_mat, sub_mat_data, 1)
+        else:
+            dispatcher = map(_quantify_sub_mat, sub_mat_data)
+        for s, result in enumerate(dispatcher):
+            cio.progress(s, n_sub_mats, f"{result['chr1']}-{result['chr2']}")
+            sub_mat_results.append(result)
 
-            # Check for nan bins (coords that do not match any Hi-C fragments
-            fall_out = np.isnan(sub_pat["bin1"]) | np.isnan(sub_pat["bin2"])
-            if np.any(fall_out):
-                n_out = len(sub_pat_bins[fall_out])
-                sys.stderr.write(
-                    f"\n{n_out} entr{'ies' if n_out > 1 else 'y'} outside "
-                    "genomic coordinates of the Hi-C matrix will be ignored.\n"
-                )
-                sub_pat = sub_pat.loc[~fall_out, :]
-            sub_pat_idx = sub_pat.index.values
-            # Convert bins from whole genome matrix to sub matrix
-            sub_pat = hic_genome.get_sub_mat_pattern(
-                mat.chr1, mat.chr2, sub_pat
-            )
-            m = mat.contact_map.matrix.tocsr()
-
-            # Feed the submatrix to quantification pipeline
-            patterns, mat_windows = cid.pattern_detector(
-                mat.contact_map,
-                cfg,
-                kernel_matrix,
-                coords=np.array(sub_pat.loc[:, ["bin1", "bin2"]]),
-                full=True,
-                tsvd=tsvd,
-            )
+        for i, r in enumerate(sub_mat_results):
+            # If there were no patterns on that sub matrix, just skip it
+            if r['coords'] is None:
+                continue
+            sub_pat_idx = sub_pos[i][0]
 
             # For each coordinate, keep the highest coefficient
             # among all kernels.
             try:
-                if kernel_id == 0:
-                    bed2d["score"][sub_pat_idx] = patterns.score.values
-                    bed2d["pvalue"][sub_pat_idx] = patterns.pvalue.values
-                    windows[sub_pat_idx, :, :] = mat_windows
-                else:
-                    # Only update scores and their corresponding windows
-                    # if better than results from previous kernels
-                    better = (
-                        patterns.score > bed2d["score"][sub_pat_idx].values
-                    ) | (np.insnan(bed2d["score"][sub_pat_idx].values))
-                    better_idx = sub_pat_idx[better]
-                    bed2d["score"][better_idx] = patterns.score[better].values
-                    windows[better_idx, :, :] = mat_windows[better]
+                bed2d_out[kernel_id]['score'][sub_pat_idx] = r['coords'].score.values
+                bed2d_out[kernel_id]["pvalue"][sub_pat_idx] = r["coords"].pvalue.values
+                windows_out[kernel_id][sub_pat_idx, :, :] = r["windows"]
             # Do nothing if no pattern was detected or matrix
             # is smaller than the kernel (-> patterns is None)
             except AttributeError:
                 pass
-            # Free space from current submatrix
-            mat.contact_map.destroy_mat()
-            del m
-            m = None
-        bed2d["bin1"] = hic_genome.coords_to_bins(
-            bed2d.loc[:, ["chrom1", "start1"]].rename(
-                columns={"chrom1": "chrom", "start1": "pos"}
-            )
+    # Select the best score for each coordinate (among the different kernels)
+    bed2d = pd.concat(bed2d_out, axis=0).reset_index(drop=True)
+    windows = np.concatenate(windows_out, axis=0)
+    bed2d = (
+        bed2d
+        .sort_values('score', ascending=True)
+        .groupby(['chrom1', 'start1', 'chrom2', 'start2'], sort=False)
+        .tail(1)
+    )
+    windows = windows[bed2d.index, :, :]
+    bed2d = bed2d.reset_index(drop=True)
+    bed2d["bin1"] = hic_genome.coords_to_bins(
+        bed2d.loc[:, ["chrom1", "start1"]].rename(
+            columns={"chrom1": "chrom", "start1": "pos"}
         )
-        bed2d["bin2"] = hic_genome.coords_to_bins(
-            bed2d.loc[:, ["chrom2", "start2"]].rename(
-                columns={"chrom2": "chrom", "start2": "pos"}
-            )
+    )
+    bed2d["bin2"] = hic_genome.coords_to_bins(
+        bed2d.loc[:, ["chrom2", "start2"]].rename(
+            columns={"chrom2": "chrom", "start2": "pos"}
         )
-        bed2d["qvalue"] = fdr_correction(bed2d["pvalue"])
-        bed2d = bed2d.loc[
-            :,
-            [
-                "chrom1",
-                "start1",
-                "end1",
-                "chrom2",
-                "start2",
-                "end2",
-                "bin1",
-                "bin2",
-                "score",
-                "pvalue",
-                "qvalue",
-            ],
-        ]
-        # Set p-values of invalid scores to nan
-        bed2d.loc[np.isnan(bed2d.score), "pvalue"] = np.nan
-        bed2d.loc[np.isnan(bed2d.score), "qvalue"] = np.nan
-        cio.write_patterns(bed2d, prefix)
-        cio.save_windows(windows, prefix, fmt=win_fmt)
-        # Generate pileup visualisations if requested
-        if plotting_enabled:
-            # Compute and plot pileup
-            pileup_title = ("pileup_of_{n}_{pattern}").format(
-                pattern=cfg["name"], n=windows.shape[0]
+    )
+    bed2d["qvalue"] = fdr_correction(bed2d["pvalue"])
+    bed2d = bed2d.loc[
+        :,
+        [
+            "chrom1",
+            "start1",
+            "end1",
+            "chrom2",
+            "start2",
+            "end2",
+            "bin1",
+            "bin2",
+            "score",
+            "pvalue",
+            "qvalue",
+        ],
+    ]
+    # Set p-values of invalid scores to nan
+    bed2d.loc[np.isnan(bed2d.score), "pvalue"] = np.nan
+    bed2d.loc[np.isnan(bed2d.score), "qvalue"] = np.nan
+    # Sort by whole genome coordinates to match input order
+    bed2d = (
+        bed2d
+        .sort_values(['bin1', 'bin2'], ascending=True)
+        .reset_index(drop=True)
+    )
+    cio.write_patterns(bed2d, prefix)
+    cio.save_windows(windows, prefix, fmt=win_fmt)
+    # Generate pileup visualisations if requested
+    if plotting_enabled:
+        # Compute and plot pileup
+        pileup_title = ("pileup_of_{n}_{pattern}").format(
+            pattern=cfg["name"], n=windows.shape[0]
+        )
+        windows_pileup = cid.pileup_patterns(windows)
+        # Symmetrize pileup for diagonal patterns
+        if not cfg["max_dist"]:
+            # Replace nan below diag by 0
+            windows_pileup = np.nan_to_num(windows_pileup)
+            # Add transpose
+            windows_pileup += np.transpose(windows_pileup) - np.diag(
+                np.diag(windows_pileup)
             )
-            windows_pileup = cid.pileup_patterns(windows)
-            # Symmetrize pileup for diagonal patterns
-            if not cfg["max_dist"]:
-                # Replace nan below diag by 0
-                windows_pileup = np.nan_to_num(windows_pileup)
-                # Add transpose
-                windows_pileup += np.transpose(windows_pileup) - np.diag(
-                    np.diag(windows_pileup)
-                )
-            pileup_plot(windows_pileup, prefix, name=pileup_title)
+        sys.stderr.write(f"Saving pileup plots in {prefix}.pdf\n")
+        pileup_plot(windows_pileup, prefix, name=pileup_title)
 
 
 def cmd_generate_config(args):
@@ -414,9 +503,11 @@ def cmd_generate_config(args):
     pattern = args["--preset"]
     click_find = args["--click"]
     n_mads = float(args["--n-mads"])
-    force_norm = args["--force-norm"]
+    norm = args["--norm"]
     win_size = args["--win-size"]
     threads = int(args["--threads"])
+    inter = args["--inter"]
+    chroms = args["--chroms"]
 
     cfg = cio.load_kernel_config(pattern, False)
 
@@ -437,21 +528,54 @@ def cmd_generate_config(args):
     # If click mode is enabled, build a kernel from scratch using
     # graphical display, otherwise, just inherit the pattern's kernel
     if click_find:
-        hic_genome = HicGenome(click_find, inter=True, kernel_config=cfg)
+        hic_genome = HicGenome(click_find, inter=inter, kernel_config=cfg)
         # Normalize (balance) the whole genome matrix
-        hic_genome.normalize(
-            force_norm=force_norm, n_mads=n_mads, threads=threads
-        )
+        hic_genome.normalize(norm=norm, n_mads=n_mads, threads=threads)
         # enforce full scanning distance in kernel config
 
         hic_genome.max_dist = hic_genome.clr.shape[0] * hic_genome.clr.binsize
         # Process each sub-matrix individually (detrend diag for intra)
         hic_genome.make_sub_matrices()
-        for sub in hic_genome.sub_mats.iterrows():
-            sub_mat = sub[1]
-            sub_mat.contact_map.create_mat()
-        processed_mat = hic_genome.gather_sub_matrices().tocsr()
-        windows = click_finder(processed_mat, half_w=int((win_size - 1) / 2))
+        # By default, the whole genome is showed at once (takes lots of RAM)
+        if chroms is None:
+            for sub in hic_genome.sub_mats.iterrows():
+                sub_mat = sub[1].contact_map
+                sub_mat.create_mat()
+            processed_mat = hic_genome.gather_sub_matrices().tocsr()
+            windows = click_finder(processed_mat, half_w=int((win_size - 1) / 2))
+        # If chromosomes were specified, their submatrices are shown one by one
+        # taking less memory (but more tedious for the user)
+        else:
+            chroms = chroms.split(',')
+            # Generate chromosome pairs to scan
+            if inter:
+                chroms = it.combinations_with_replacement(chroms, 2)
+            else:
+                chroms = [(ch, ch) for ch in chroms]
+            windows = []
+            for c1, c2 in chroms:
+                try:
+                    sub_mat = hic_genome.sub_mats.query(
+                            '(chr1 == @c1) & (chr2 == @c2)'
+                    )['contact_map'].values[0]
+                # In case chromosomes have been entered in a different order
+                except IndexError:
+                    c1, c2 = c2, c1
+                    sub_mat = hic_genome.sub_mats.query(
+                            '(chr1 == @c1) & (chr2 == @c2)'
+                    )['contact_map'].values[0]
+                sub_mat.create_mat()
+                chrom_wins = click_finder(
+                        sub_mat.matrix.tocsr(),
+                        half_w=int((win_size - 1) / 2),
+                        xlab=c2,
+                        ylab=c1
+                )
+                windows.append(chrom_wins)
+                sub_mat.destroy_mat()
+            windows = np.concatenate(windows, axis=0)
+
+
         # Pileup all recorded windows and convert to JSON serializable list
         pileup = ndi.gaussian_filter(cid.pileup_patterns(windows), 1)
         cfg["kernels"] = [pileup.tolist()]
@@ -502,7 +626,7 @@ def _detect_sub_mat(data):
 def cmd_detect(args):
     # Parse command line arguments for detect
     dump = args["--dump"]
-    force_norm = args["--force-norm"]
+    norm = args["--norm"]
     interchrom = args["--inter"]
     iterations = args["--iterations"]
     kernel_config_path = args["--kernel-config"]
@@ -586,7 +710,7 @@ def cmd_detect(args):
     ### 1: Process input signal
     hic_genome.kernel_config = cfg
     # Normalize (balance) matrix using ICE
-    hic_genome.normalize(force_norm=force_norm, n_mads=n_mads, threads=threads)
+    hic_genome.normalize(norm=norm, n_mads=n_mads, threads=threads)
     # Define how many diagonals should be used in intra-matrices
     hic_genome.compute_max_dist()
     # Split whole genome matrix into intra- and inter- sub matrices. Each sub
@@ -628,9 +752,7 @@ def cmd_detect(args):
             else:
                 dispatcher = map(_detect_sub_mat, sub_mat_data)
             for s, result in enumerate(dispatcher):
-                chr1 = hic_genome.sub_mats.chr1[s]
-                chr2 = hic_genome.sub_mats.chr2[s]
-                cio.progress(s, n_sub_mats, f"{chr1}-{chr2}")
+                cio.progress(s, n_sub_mats, f"{result['chr1']}-{result['chr2']}")
                 sub_mat_results.append(result)
 
             # Convert coordinates from chromosome to whole genome bins
@@ -716,7 +838,13 @@ def cmd_detect(args):
     )
     all_coords = all_coords.loc[~min_dist_drop_mask, :]
     all_windows = all_windows[~min_dist_drop_mask, :, :]
+    del min_dist_drop_mask
 
+    # Remove patterns with nan p-values (no contact in window)
+    pval_mask = np.isnan(all_coords.pvalue)
+    all_coords = all_coords.loc[~pval_mask, :]
+    all_windows = all_windows[~pval_mask, :, :]
+    del pval_mask
     # Correct p-values for multiple testing using FDR
     all_coords["qvalue"] = fdr_correction(all_coords["pvalue"])
     # Reorder columns
@@ -742,8 +870,12 @@ def cmd_detect(args):
     ### 3: WRITE OUTPUT
     sys.stderr.write(f"{all_coords.shape[0]} patterns detected\n")
     # Save patterns and their coordinates in a tsv file
+
+    sys.stderr.write(f"Saving patterns in {prefix}.tsv\n")
     cio.write_patterns(all_coords, prefix)
     # Save windows as an array in an npy file
+
+    sys.stderr.write(f"Saving patterns in {prefix}.{win_fmt}\n")
     cio.save_windows(all_windows, prefix, fmt=win_fmt)
 
     # Generate pileup visualisations if requested
@@ -761,9 +893,39 @@ def cmd_detect(args):
             windows_pileup += np.transpose(windows_pileup) - np.diag(
                 np.diag(windows_pileup)
             )
+        sys.stderr.write(f"Saving pileup plots in {prefix}.pdf\n")
         pileup_plot(windows_pileup, prefix, name=pileup_title)
 
 
+def cmd_list_kernels(args):
+
+    kernel_name = args["--name"]
+    # Load every avaiable kernel by default
+    if kernel_name == "all":
+        kernels = ck.kernel_names
+    # If a specific kernel was requested, only load this one
+    else:
+        kernels = [kernel_name]
+
+    # Check availability of each kernel and print its name
+    for k in kernels:
+        try:
+            kernel_infos = getattr(ck, k)
+        except AttributeError:
+            raise ValueError(f"Kernel {k} is not available")
+        print(k)
+        # Print default params if --long specified (key-value pairs in json)
+        if args["--long"]:
+            exclude_params = ["name", "resolution", "kernels"]
+            for param, value in kernel_infos.items():
+                if param not in exclude_params:
+                    print(f"  {param}: {value}")
+        if args["--mat"]:
+            mats = kernel_infos["kernels"]
+            for mat in mats:
+                print_ascii_mat(mat)
+
+
 def cmd_test(args):
 
     sys.stderr.write(f"Fetching test dataset at {URL_EXAMPLE_DATASET}...\n")
@@ -773,7 +935,7 @@ def cmd_test(args):
     sys.stderr.write(f"Running detection on test dataset...\n")
 
     args["<contact_map>"] = tmp_cool.name
-    args["<prefix>"] = 'chromosight_test'
+    args["<prefix>"] = "chromosight_test"
     args["--no-plotting"] = True
     cmd_detect(args)
     os.unlink(tmp_cool.name)
@@ -797,10 +959,18 @@ def capture_ouput(stderr_to=None):
             pass
 
 
+def logo_version(logo, ver):
+    small_logo = resize_kernel(logo, factor=0.33, quiet=True)
+    ascii_logo = print_ascii_mat(small_logo, colored=False, print_str=False)
+    return f"{ascii_logo} Chromosight version {ver}"
+
+
 def main():
-    args = docopt.docopt(__doc__, version=__version__)
+
+    args = docopt.docopt(__doc__, version=logo_version(LOGO, __version__))
     detect = args["detect"]
     generate_config = args["generate-config"]
+    list_kernels = args["list-kernels"]
     quantify = args["quantify"]
     test = args["test"]
     if test:
@@ -831,8 +1001,11 @@ def main():
         cmd_detect(args)
     elif generate_config:
         cmd_generate_config(args)
+    elif list_kernels:
+        cmd_list_kernels(args)
     elif quantify:
         cmd_quantify(args)
+
     return 0
 
 
