@@ -134,6 +134,8 @@ class HicGenome:
         self.detectable_bins = np.array(range(self.clr.shape[0]))
         self.inter = inter
         self.compute_max_dist()
+        # Whether normalized or raw matrices should be used
+        self.use_norm = True
         if sample is not None:
             sample = float(sample)
             try:
@@ -177,7 +179,7 @@ class HicGenome:
             self.max_dist = None
             self.largest_kernel = 3
 
-    def normalize(self, force_norm=False, n_mads=5, threads=1):
+    def normalize(self, norm='auto', n_mads=5, threads=1):
         """
         If the instance's cooler is not balanced, finds detectable bins and
         applies ICE normalisation to the whole genome matrix. Newly computed
@@ -186,9 +188,11 @@ class HicGenome:
 
         Parameters
         ----------
-        force_norm : bool
-            Whether to force recomputing of or reuse existing weights in the
-            cool file.
+        force_norm : str
+            Normalization behaviour. If 'auto', existing weights are reused and
+            matrix is balanced only in the absence of weights. If 'raw', raw
+            contact values are used. If 'force', weights are recomputed and the
+            underlying cool file is overwritten.
         n_mads : float
             Maximum number of median absoluted deviations (MADs) below the
             median of the distribution of logged bin sums to consider a bin
@@ -196,7 +200,9 @@ class HicGenome:
         threads : int
             Number of parallel threads to use for normalization.
         """
-        if "weight" in self.bins.columns and not force_norm:
+        if norm not in ['auto', 'raw', 'force']:
+            raise ValueError("norm must be one of: auto, raw, force")
+        if "weight" in self.bins.columns and norm != 'force':
             sys.stderr.write("Matrix already balanced, reusing weights\n")
         else:
             pool = mp.Pool(threads)
@@ -215,6 +221,10 @@ class HicGenome:
             print("Whole genome matrix balanced")
             # Reload bins attribute to include the weight  column
             self.bins = self.clr.bins()[:]
+        if norm == 'raw':
+            self.use_norm = False
+        else:
+            self.use_norm = True
         # Bins with NaN weight are missing, matrix already balanced
         self.detectable_bins = np.flatnonzero(np.isfinite(self.bins.weight))
         print(
@@ -261,7 +271,6 @@ class HicGenome:
                 f"{np.round(100 * self.sample)}% contacts will be sampled \n"
             )
         sub_mat_idx = 0
-        # mat_view = self.clr.matrix(sparse=True, balance=True)
         for i1, chr1 in enumerate(self.clr.chromnames):
             for i2, chr2 in enumerate(self.clr.chromnames):
                 s1, e1 = self.clr.extent(chr1)
@@ -275,28 +284,28 @@ class HicGenome:
                         d[(d >= s1) & (d < e1)] - s1,
                         d[(d >= s2) & (d < e2)] - s2,
                     )
+                    map_kwargs = {
+                        'smooth': self.smooth,
+                        'sample': self.sample,
+                        'dump': self.dump,
+                        'use_norm': self.use_norm,
+                        'extent': [(s1, e1), (s2, e2)],
+                        'detectable_bins': sub_mat_detectable_bins,
+                        'name': f"{chr1}-{chr2}",
+                    }
                     if i1 == i2:
                         sub_mats.contact_map[sub_mat_idx] = ContactMap(
                             self.clr,
-                            extent=[(s1, e1), (s2, e2)],
-                            detectable_bins=sub_mat_detectable_bins,
                             inter=False,
                             max_dist=self.max_dist,
                             largest_kernel=self.largest_kernel,
-                            dump=self.dump,
-                            name=f"{chr1}-{chr2}",
-                            smooth=self.smooth,
-                            sample=self.sample,
+                            **map_kwargs,
                         )
                     else:
                         sub_mats.contact_map[sub_mat_idx] = ContactMap(
                             self.clr,
-                            extent=[(s1, e1), (s2, e2)],
-                            detectable_bins=sub_mat_detectable_bins,
                             inter=True,
-                            dump=self.dump,
-                            name=f"{chr1}-{chr2}",
-                            sample=self.sample,
+                            **map_kwargs,
                         )
                     sub_mats.chr1[sub_mat_idx] = chr1
                     sub_mats.chr2[sub_mat_idx] = chr2
@@ -453,6 +462,8 @@ class ContactMap:
     extent : list of tuples of ints
         List of two tuples containing the start and end bin numbers of both
         chromosomes from the submatrix.
+    matrix : scipy.sparse.csr_matrix
+        The contact map as a sparse matrix.
     detectable_bins : tuple of arrays
         List containing two arrays (rows and columns) of indices from bins
         considered detectable in the matrix.
@@ -474,6 +485,9 @@ class ContactMap:
     sample : int, float or None
         Proportion of contacts to sample from the data if between 0 and 1.
         Number of contacts to keep if above 1. Keep all if None.
+    use_norm : bool
+        Whether to use the balanced matrix. If set to False, the raw contact
+        counts are used.
     """
 
     def __init__(
@@ -488,6 +502,7 @@ class ContactMap:
         dump=None,
         smooth=False,
         sample=None,
+        use_norm=True,
     ):
         self.clr = clr
         self.extent = extent
@@ -500,6 +515,7 @@ class ContactMap:
         self.largest_kernel = largest_kernel
         self.dump = dump
         self.matrix = None
+        self.use_norm = use_norm
         # If detectable were not provided, compute them from the input matrix
         if detectable_bins is None:
             detectable_bins = preproc.get_detectable_bins(
@@ -510,18 +526,26 @@ class ContactMap:
 
     def create_mat(self):
         (s1, e1), (s2, e2) = self.extent
-        self.matrix = self.clr.matrix(sparse=True, balance=True)[s1:e1, s2:e2]
-        # Remove NaN values to store them as implicit zeroes
-        self.matrix.data[np.isnan(self.matrix.data)] = 0
-        self.matrix.eliminate_zeros()
+        self.matrix = self.clr.matrix(sparse=True, balance=self.use_norm)[s1:e1, s2:e2]
         # Subsample contacts if requested
         if self.sample is not None:
-            self.subsample(self.sample)
+            self.subsample(self.sample, balance=self.use_norm)
         # Apply preprocessing steps on the input matrix
         if self.inter:
             self.preprocess_inter_matrix()
         else:
             self.preprocess_intra_matrix()
+        # Remove NaN values to store them as implicit zeroes
+        if self.use_norm:
+            self.matrix.data[np.isnan(self.matrix.data)] = 0
+        # Raw matrices do not have nan values, we need to use the weights
+        # to deduce missing bins
+        else:
+            self.matrix = self.matrix.tocsr()
+            self.matrix[preproc.valid_to_missing(self.detectable_bins[0], self.matrix.shape[0]), :] = 0
+            self.matrix[:, preproc.valid_to_missing(self.detectable_bins[1], self.matrix.shape[1])] = 0
+            self.matrix = self.matrix.tocoo()
+        self.matrix.eliminate_zeros()
 
     def destroy_mat(self):
         """Destroys contact map to clean up memory"""
@@ -570,11 +594,11 @@ class ContactMap:
                 bias_cols[self.matrix.col] *
                 self.matrix.data
                 )
-            self.matrix.data[np.isnan(self.matrix.data)] = 0
 
     @DumpMatrix("01_process_inter")
     def preprocess_inter_matrix(self):
-        self.matrix /= np.median(self.matrix.data)
+        self.matrix.data[np.isnan(self.matrix.data)] = 0.0
+        self.matrix.data = self.matrix.data / np.nanmedian(self.matrix.data)
 
     def preprocess_intra_matrix(self):
         self.detrend()
@@ -588,6 +612,7 @@ class ContactMap:
             max_dist=self.keep_distance,
             smooth=self.smooth,
             detectable_bins=self.detectable_bins[0],
+            max_val=10 if self.use_norm else None,
         )
 
     @DumpMatrix("02_remove_diags")
