@@ -223,6 +223,7 @@ def pattern_detector(
     km, kn = kernel_matrix.shape
     kh, kw = (km - 1) // 2, (kn - 1) // 2
 
+
     def save_dump(base, mat):
         """Define where to save the dump"""
         sp.save_npz(
@@ -275,8 +276,8 @@ def pattern_detector(
     # Only attempt detection if no input coordinates were given
     if run_mode == "detect":
         # Find foci of highly correlated pixels and pick local maxima
-        # coords, foci_mat = picker(np.abs(mat_log10_pvals), 5)
-        coords, foci_mat = picker(mat_conv, kernel_config["pearson"],)
+        # coords, foci_mat = pick_foci(np.abs(mat_log10_pvals), 5)
+        coords, foci_mat = pick_foci(mat_conv, kernel_config["pearson"],)
         # If nothing was detected, no point in resuming
         if coords is None:
             return None, None
@@ -383,7 +384,7 @@ def remove_neighbours(patterns, win_size=8):
     return whitelist_mask
 
 
-def picker(mat_conv, pearson):
+def pick_foci(mat_conv, pearson, min_size=2):
     """
     Pick coordinates of local maxima in a sparse 2D convolution heatmap. A
     threshold computed based on the pearson argument is applied to the heatmap.
@@ -398,6 +399,9 @@ def picker(mat_conv, pearson):
         Minimum correlation coefficient required to consider a pixel as
         candidate. Increasing this value reduces the amount of false
         positive patterns.
+    min_size : int
+        Minimum number of pixels required to keep a focus. Pixels belonging to
+        smaller foci will be set to 0.
     Returns
     -------
     foci_coords : numpy.array of ints
@@ -421,7 +425,7 @@ def picker(mat_conv, pearson):
         # Assign foci identifiers to pixels above threshold
         num_foci, labelled_mat = label_foci(candidate_mat)
         # Remove foci which are too small
-        num_foci, labelled_mat = filter_foci(labelled_mat)
+        num_foci, labelled_mat = filter_foci(labelled_mat, min_size=min_size)
         if num_foci == 0:
             return None, None
         mat_conv = mat_conv.tocsr()
@@ -686,13 +690,27 @@ def _xcorr2_sparse(signal, kernel, threshold=1e-6):
                 np.ones(kn), -np.arange(kn), shape=(sn, sn - kn + 1), format="dia",
             )
             out = (l_subkernel_sp @ signal) @ r_subkernel_sp
-        # Convolution code for general case
+        # convolution code for general case
+        # 1. 2D kernel composed of 1D filters, each col being a 1D filter.
+        # 2. input remains unchanged, and each 1D kernel is unrolled into
+        # a sparse toeplitz matrix.
+        # 3. Each 1D conv is computed via a sparse matrix x vector product.
+        # It is fast because the product is delegated to numpy.
         else:
-            for kj in range(kn):
-                subkernel_sp = sp.diags(
-                    kernel[:, kj], np.arange(km), shape=(sm - km + 1, sm), format="csr",
-                )
-                out += subkernel_sp.dot(signal[:, kj : sn - kn + 1 + kj])
+            # In case the kernel is rectangle, it is faster to scan
+            # the largest dimension first
+            if kn < km:
+                for kj in range(kn):
+                    subkernel_sp = sp.diags(
+                        kernel[:, kj], np.arange(km), shape=(sm - km + 1, sm), format="csr",
+                    )
+                    out += subkernel_sp.dot(signal[:, kj : sn - kn + 1 + kj])
+            else:
+                for ki in range(km):
+                    subkernel_sp = sp.diags(
+                        np.array(kernel[ki, :]).flatten(), np.arange(kn), shape=(sn - kn + 1, sn), format="csr",
+                    )
+                    out += signal[ki : sm - km + 1 + ki, :].dot(subkernel_sp.T)
 
     # Set very low pixels to 0
     out.data[np.abs(out.data) < threshold] = 0
@@ -761,11 +779,19 @@ def _xcorr2_dense(signal, kernel, threshold=1e-6):
             )
             out_wo_margin = (l_subkernel_sp @ signal) @ r_subkernel_sp
         # convolution code for general case
+        # 1. 2D kernel composed of 1D filters, each col being a 1D filter.
+        # 2. input remains unchanged, and each 1D kernel is unrolled into
+        # a sparse toeplitz matrix.
+        # 3. Each 1D conv is computed via a sparse matrix x vector product.
+        # It is fast because the product is delegated to numpy.
         else:
             for kj in range(kn):
+                # Make a toeplitz matrix from the kj'th column of the kernel
                 subkernel_sp = sp.diags(
                     kernel[:, kj], np.arange(km), shape=(sm - km + 1, sm), format="csr",
                 )
+                # Note: We compute the dot product of the subkernel with the signal 
+                # shifted by kj.
                 out_wo_margin += subkernel_sp @ signal[:, kj : sn - kn + 1 + kj]
 
     kh = (km - 1) // 2
