@@ -279,8 +279,8 @@ def detrend(
         Whether to use isotonic regression to smooth the trend.
     fun : callable
         Function to use on each diagonal to compute the trend.
-    max_val : float
-        Maximum value in the detrended matrix.
+    max_val : float or None
+        Maximum value in the detrended matrix. Set to None to disable
 
     Returns
     -------
@@ -296,17 +296,17 @@ def detrend(
         fun=fun,
     )
     y[np.isnan(y)] = 0.0
-
     # Detrending by the distance law
     clean_mat = matrix.tocoo()
     # clean_mat.data /= y_savgol[abs(clean_mat.row - clean_mat.col)]
     try:
-        clean_mat.data /= y[abs(clean_mat.row - clean_mat.col)]
+        clean_mat.data = clean_mat.data / y[abs(clean_mat.row - clean_mat.col)]
     # If no nonzero value in matrix, do nothing
     except TypeError:
         pass
     clean_mat = clean_mat.tocsr()
-    clean_mat[clean_mat >= max_val] = max_val
+    if max_val is not None:
+        clean_mat[clean_mat >= max_val] = 1
     return clean_mat
 
 
@@ -734,6 +734,7 @@ def resize_kernel(
     signal_res=None,
     factor=None,
     min_size=7,
+    quiet=False,
 ):
     """
     Resize a kernel matrix based on the resolution at which it was defined and
@@ -759,6 +760,8 @@ def resize_kernel(
         shrink it.
     min_size : int
         Lower bound, in number of rows/column allowed when resizing the kernel.
+    quiet : bool
+        Suppress warnings if resize factor was adjusted.
 
     Returns
     -------
@@ -795,9 +798,10 @@ def resize_kernel(
     if not resized_kernel.shape[0] % 2:
         # Compute the factor required to yield a dimension smaller by one
         adj_resize_factor = (resized_kernel.shape[0] - 1) / km
-        sys.stderr.write(
-            f"Adjusting resize factor from {resize_factor} to {adj_resize_factor}.\n"
-        )
+        if not quiet:
+            sys.stderr.write(
+                f"Adjusting resize factor from {resize_factor} to {adj_resize_factor}.\n"
+            )
         resized_kernel = ndi.zoom(kernel, adj_resize_factor, order=1)
 
     return resized_kernel
